/*
 * @Author: Chi Yao 
 * @Date: 2019-01-19 12:21:34 
 * @Last Modified by: Chi Yao
 * @Last Modified time: 2019-02-23 09:28:29
 */

var request = require("request");
var Crypto = require("../utils/CryptoUtil");
var Config = require("../Config");
const callAPI = function (options) {
    var timestamp = new Date().getTime();
    options.data = options.data || {};
    options.data._time = timestamp;
    request({
        url: "/openApi?_time=" + timestamp,
        method: options.method || "POST",
        baseUrl: Config.api.http.url,
        timeout: options.timeout || 9 * 1000,
        headers: {
            'KTT-CSID': Config.keys.csid,
        },
        json: options.data
    }, (err, packet, jsonbody) => {
        if (!err && jsonbody.code == "000000") {
            options.success && options.success(jsonbody);
        } else {
            options.error && options.error(err, packet, jsonbody);
        }
        options.complete && options.complete();
    });
}
const callAPIMethod = function (method, targetInfo, params, done) {
    if (typeof params == "function") {
        done = params;
        params = undefined;
    }
    var optionData = {
        method: method,
        time: new Date().getTime()
    };
    if (targetInfo) {
        optionData.targetDevice = targetInfo.deviceId;
        optionData.token = targetInfo.token;
    }
    if (params) {
        optionData.params = params;
    }
    var options = {};
    if (params && params.options) {
        options = params.options;
        delete params.options;
    }

    callAPI(Object.assign(options, {
        data: optionData,
        success: (data) => {
            done && done(undefined, data.data);
        },
        error: (err, packet, jsonbody) => {
            if (err) {
                done && done(err);
            } else if (jsonbody) {
                let error = new Error(jsonbody.desc);
                error.code = jsonbody.code;
                done && done(error);
            } else {
                done && done(err || new Error("999999"));
            }
        }
    }));
}
const Manage = {
    addHub: (hubInfo, next) => {
        if (!hubInfo.hubId || !hubInfo.hubPwd) {
            next(new Error(900001));
        } else {
            if (hubInfo.hubPwd.length != 32) {
                hubInfo.hubPwd = Crypto.encryptHash(hubInfo.hubPwd);
            }
            hubInfo.hubPwd = hubInfo.hubPwd.toLowerCase();
            callAPIMethod("Manage.addHub", undefined, hubInfo, next);
        }
    },
    searchDevice: (hubInfo, next) => {
        if (!hubInfo.deviceId || !hubInfo.token) {
            next(new Error(900001));
        } else {
            callAPIMethod("Manage.searchDevice", hubInfo, { options: { timeout: 30 * 1000 } }, next);
        }
    },
    getAddedDevice:(hubInfo,next)=>{
        callAPIMethod("Manage.getAddedDevices", undefined,{hubId:hubInfo.deviceId}, next);
    }
}
const Hub = {
    getGeneralInfo:(hubInfo,next)=>{
        if (!hubInfo.deviceId || !hubInfo.token) {
            next(new Error(900001));
        } else {
            callAPIMethod("Hub.getGeneralInfo", hubInfo, undefined, next);
        }
    }
}
const Thermostat = {
    getState:(deviceInfo,next)=>{
        if (!deviceInfo.deviceId || !deviceInfo.token) {
            next(new Error(900001));
        } else {
            callAPIMethod("Thermostat.getState", deviceInfo, undefined, next);
        }
    },
    setState:(deviceInfo,params,next)=>{
        if (!deviceInfo.deviceId || !deviceInfo.token) {
            next(new Error(900001));
        } else {
            callAPIMethod("Thermostat.setState", deviceInfo, params, next);
        }
    }
}
exports.Manage = Manage;
exports.Hub = Hub;
exports.Thermostat = Thermostat;