/*
 * @Author: Chi Yao 
 * @Date: 2019-01-19 12:21:34 
 * @Last Modified by: Chi Yao
 * @Last Modified time: 2019-11-12 20:03:32
 */

var request = require("request");
var Crypto = require("../utils/CryptoUtil");
var Config = require("../Config");
const callAPI = function (options) {
    options.data = options.data || {};
    if (!options.data.time) {
        options.data.time = new Date().getTime();
    }
    let _body = JSON.stringify(options.data);
    request({
        url: "/openApi",
        method: options.method || "POST",
        baseUrl: Config.api.http.url,
        timeout: options.timeout || 10 * 1000,
        headers: {
            'YS-CSID': Config.keys.csid,
            'YS-SEC': Crypto.encryptHash(_body + Config.keys.cskey),
            'Content-Type': 'application/json',
        },
        body: _body
    }, (err, packet, jsonbody) => {
        if (!err && typeof jsonbody == 'string') {
            try {
                jsonbody = JSON.parse(jsonbody);
            } catch (e) {
            }
            if (jsonbody.code == "000000") {
                options.success && options.success(jsonbody);
            } else {
                options.error && options.error(err, packet, jsonbody);
            }

        } else {
            options.error && options.error(err, packet, jsonbody);
        }
        options.complete && options.complete();
    });
}
const callAPIMethod = function (method, targetInfo, params, done) {
    if (typeof params == "function") {
        done = params;
        params = undefined;
    }
    var optionData = {
        method: method,
        time: new Date().getTime()
    };
    if (targetInfo) {
        optionData.targetDevice = targetInfo.deviceId;
        optionData.token = targetInfo.token;
    }
    if (params) {
        optionData.params = params;
    }
    var options = {};
    if (params && params.options) {
        options = params.options;
        delete params.options;
    }

    callAPI(Object.assign(options, {
        data: optionData,
        success: (data) => {
            done && done(undefined, data.data);
        },
        error: (err, packet, jsonbody) => {
            if (err) {
                done && done(err);
            } else if (jsonbody) {
                let error = new Error(jsonbody.desc);
                error.code = jsonbody.code;
                done && done(error);
            } else {
                done && done(err || new Error("999999"));
            }
        }
    }));
}
const Manage = {
    addYoLinkDevice: (sn, next) => {
        if (!sn) {
            next(new Error(900001));
        } else {
            callAPIMethod("Manage.addYoLinkDevice", undefined, { sn }, next);
        }
    },
}
const InfraredRemoter = {
    send: (deviceInfo, key, next) => {
        if (!deviceInfo.deviceId || !deviceInfo.token) {
            next(new Error(900001));
        } else {
            callAPIMethod("InfraredRemoter.send", deviceInfo, { key }, next);
        }
    }
}
exports.Manage = Manage;
exports.InfraredRemoter = InfraredRemoter;