/*
 * @Author: Chi Yao 
 * @Date: 2019-01-19 12:21:39 
 * @Last Modified by: Chi Yao
 * @Last Modified time: 2019-02-23 09:51:02
 */
const mqtt = require("mqtt");
const Config = require("../Config");
const UUID = require('uuid');
const CallbakckProxy = require("../utils/SimpleCallbackProxy")
function Client(ch_fix) {
	this.serverURL = Config.api.mqtt.url;
	this.username = Config.api.mqtt.user;
	this.password = Config.api.mqtt.pwd
	this.clientId = UUID.v1();
	this.ch_fix = ch_fix;
	this.pubChannel = `${Config.keys.name}${this.ch_fix?("/"+this.ch_fix):""}/request`;
	this.subChannel = `${Config.keys.name}${this.ch_fix?("/"+this.ch_fix):""}/response`;
	this.repChannel = `${Config.keys.name}${this.ch_fix?("/"+this.ch_fix):""}/report`;
	this.callbackProxy = new CallbakckProxy();
}
Client.prototype.connect = function (done) {
	this.mqttClient = mqtt.connect(this.serverURL,
		{
			username: this.username,
			password: this.password,
			clientId: this.clientId
		});
	this.mqttClient.on("connect",()=> {
		this.mqttClient.subscribe([this.subChannel,this.repChannel],()=>{
			done&&done();
		});
	});
	this.mqttClient.on("message", (channel, payload)=> {
		try {
			var message = JSON.parse(payload);
			console.log(message);
			if (channel == this.repChannel) {
				//Do Report Logic Here
			}
			if(channel == this.subChannel){
				process.nextTick(()=>{
					this.callbackProxy.triggerItem(message.msgid,message);
				});
			}
		} catch (e) {
			console.error(e);
		}
	});
}
Client.prototype.disconnect=function(done){
	this.mqttClient.end(false,done);
}
Client.prototype.publish = function (method,targetInfo,params, call) {
	var self = this;
	let message = {
        method
    };
    if (targetInfo) {
        message.targetDevice = targetInfo.deviceId;
        message.token = targetInfo.token;
    }
    if (params) {
        message.params = params;
	}
	this.sendGeneralMessage(message,call);
}
Client.prototype.sendGeneralMessage = function ( message, call) {
	if(!message.time){
		message.time=new Date().getTime();
	}
	if (!message.msgid) {
		message.msgid = message.time;
	}
	this.sendMQTTMessage(this.pubChannel,message,  (err)=> {
		if (err) {
			console.log(err);
			call && call(new Error("999999"));
		} else {
			this.callbackProxy.addItem(message.msgid,(err,jsonbody)=>{
				if (!err && jsonbody.code == "000000") {
					call&&call(undefined,jsonbody);
				} else {
					if(!err){
						err = new Error(jsonbody.desc);
						err.code = jsonbody.code;
					}
					call&&call(err);
				}
			});
		}
	});
}
Client.prototype.sendMQTTMessage = function (topic, message, call) {
	if (typeof message == "string") {
		this.mqttClient.publish(topic, message, call);
	} else if (typeof message == "object") {
		console.log(`Publish ${JSON.stringify(message)} at ${topic}`);
		this.mqttClient.publish(topic, JSON.stringify(message), call);
	} else {
		call(new Error("000102", message));
	}
}
module.exports = Client;
